==> menta-queue-0.9.2-sources.jar: org/mentaqueue/BlockingLinkedQueue.java <==
/*
 * MentaQueue => http://mentaqueue.soliveirajr.com
 * Copyright (C) 2012  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentaqueue;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.mentaqueue.util.Builder;

public class BlockingLinkedQueue<E> implements Queue<E> {

	private final static int DEFAULT_CAPACITY = 1026 * 16;

	private final java.util.concurrent.ConcurrentLinkedQueue<E> pool;
	private final List<E> polledElements;
	private final LinkedList<E> queue;
	private final Object lock = new Object();
	private final int capacity;

	public BlockingLinkedQueue(int capacity, Builder<E> builder) {
		pool = new java.util.concurrent.ConcurrentLinkedQueue<E>();
		polledElements = new LinkedList<E>();
		queue = new LinkedList<E>();
		this.capacity = capacity;
		for (int i = 0; i < capacity; i++) {
			pool.offer(builder.newInstance());
		}
	}

	public BlockingLinkedQueue(Builder<E> builder) {
		this(DEFAULT_CAPACITY, builder);
	}
	
	private final static <E> Builder<E> createBuilder(final Class<E> klass) {
		return new Builder<E>() {
			@Override
            public E newInstance() {
	            try {
	            	return klass.newInstance();
	            } catch(Exception e) {
	            	throw new RuntimeException(e);
	            }
            }
		};
	}
	
	public BlockingLinkedQueue(Class<E> klass) {
		this(createBuilder(klass));
	}
	
	public BlockingLinkedQueue(int capacity, Class<E> klass) {
		this(capacity, createBuilder(klass));
	}


	@Override
	public E nextToOffer() {
		return pool.poll();
	}

	@Override
	public void offer(E e) {
		synchronized(lock) {
			int size = queue.size();
			if (size == capacity) {
				try {
					lock.wait();
				} catch(Exception exc) {
					return;
				}
			}
			queue.add(e);
			if (size == 0) {
				lock.notify();
			}
		}
	}

	@Override
	public long available() {
		synchronized(lock) {
			if (queue.size() == 0) {
				try {
					lock.wait();
				} catch(Exception e) {
					return 0;
				}
			}
			return queue.size();
		}
	}

	@Override
	public E poll() {
		E elem;
		synchronized(lock) {
			elem = queue.removeFirst();
			if (queue.size() == capacity - 1) {
				lock.notify();
			}
		}
		polledElements.add(elem);
		return elem;
	}

	@Override
	public void done() {
		Iterator<E> iter = polledElements.iterator();
		while(iter.hasNext()) {
			pool.offer(iter.next());
			iter.remove();
		}
	}
}

==> menta-queue-0.9.3-sources.jar: org/mentaqueue/BlockingLinkedQueue.java <==
/*
 * MentaQueue => http://mentaqueue.soliveirajr.com
 * Copyright (C) 2012  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentaqueue;

import static org.mentaqueue.util.BuilderUtils.*;

import org.mentaqueue.util.Builder;

public class BlockingLinkedQueue<E> implements Queue<E> {

	private final static int DEFAULT_CAPACITY = 1026 * 16;

	private final java.util.concurrent.LinkedBlockingQueue<E> queue;
	private final Builder<E> builder;

	public BlockingLinkedQueue(int capacity, Builder<E> builder) {
		queue = new java.util.concurrent.LinkedBlockingQueue<E>(capacity);
		this.builder = builder;
	}

	public BlockingLinkedQueue(Class<E> klass) {
		this(DEFAULT_CAPACITY, createBuilder(klass));
	}
	
	public BlockingLinkedQueue(Builder<E> builder) {
		this(DEFAULT_CAPACITY, builder);
	}
	
	@Override
	public E nextToOffer() {
		return builder.newInstance();
	}

	@Override
	public void offer(E e) {
		try {
			queue.put(e);
		} catch(Exception exc) {
			throw new RuntimeException(exc);
		}
	}

	@Override
	public long available() {
		return queue.size();
	}

	@Override
	public E poll() {
		return queue.poll();
	}

	@Override
	public void done() {
	}
}
